/*
   Class Name:

      CDDSLoader.

   Created by:

      Allen Sherrod (Programming Ace of www.UltimateGameProgramming.com).

   Description:

      This class loads a dds texture into this object.
*/


#include"CDDSLoader.h"
#include<ddraw.h>


CDDSLoader::CDDSLoader()
{
   // Give everything default values.
   image = 0;
   type = 0;
   ID = -1;
   imageWidth = 0;
   imageHeight = 0;
   totalComponenets = 0;
   totalMips = 0;
}


CDDSLoader::~CDDSLoader()
{
   FreeImage();
}


bool CDDSLoader::LoadDDSFile(char *file)
{
   FreeImage();

   FILE *fp;
   DDSURFACEDESC2 ddsd;
   char imageID[4];
   int mipFactor = 0;
   int totalSize = 0;
   int imageSize = 0;

   fp = fopen(file, "rb");
   if(!fp) return false;

   // Check if the image ID is a .dds file.
   fread(imageID, 1, 4, fp);

   if(strncmp(imageID, "DDS ", 4) != 0)
      {
         fclose(fp);
         return false;
      }

   // Read the surface information.
   fread(&ddsd, sizeof(DDSURFACEDESC2), 1, fp);
   
   // Make sure it is not 0 in size.
   if(ddsd.dwLinearSize != 0)
      {
         imageSize = ddsd.dwLinearSize;
      }
   else
      {
         imageSize = ddsd.dwHeight * ddsd.dwWidth;
      }

   // Error checking.
   if(imageSize <= 0)
      {
         fclose(fp);
         return false;
      }

   totalMips = ddsd.dwMipMapCount;   
   imageWidth = ddsd.dwWidth;
   imageHeight = ddsd.dwHeight;

   // DXT1 has a compression ratio of 8:1, DXT3 has 4:1, and DXT5 has 4:1.
   switch(ddsd.ddpfPixelFormat.dwFourCC)
      {
         case FOURCC_DXT1:
            mipFactor = 2;
            totalComponenets = 3;
            type = GL_COMPRESSED_RGBA_S3TC_DXT1_EXT;
            break;

         case FOURCC_DXT3:
            mipFactor = 4;
            totalComponenets = 4;
            type = GL_COMPRESSED_RGBA_S3TC_DXT3_EXT;
            break;

         case FOURCC_DXT5:
            mipFactor = 4;
            totalComponenets = 4;
            type = GL_COMPRESSED_RGBA_S3TC_DXT5_EXT;
            break;

         default:
            fclose(fp);
            return false;
            break;
      }

   // If this image has more than the one mip map level then take that into
   // account, else it is just the one mip map level in the image.
   if(ddsd.dwMipMapCount > 1) totalSize = imageSize * mipFactor;
   else totalSize = imageSize;

   // Allocate memory for every mip map level.
   image = new unsigned char[totalSize * sizeof(unsigned char)];
   
   // Make sure the image loaded.
	if(image == NULL)
	   {
	      fclose(fp);
         return false;
	   }

   // Read the entire image data.
   fread(image, 1, totalSize, fp);

   fclose(fp);
   return true;
}


void CDDSLoader::FreeImage()
{
   if(image)
      {
         delete[] image;
         image = 0;
         type = 0;
         imageWidth = 0;
         imageHeight = 0;
         totalComponenets = 0;
         totalMips = 0;
      }
}


// Copyright October 2004
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com