/*
   .dds Loader Demo.
   Created by the Programming Ace.
   www.UltimateGameProgramming.com

   The purpose of this demo is to load a .dds image file into OpenGL.  A .dds image is
   actually a Direct Draw (part of DirectX) surface.  .dds images can be compessed using
   DXT1, DXT3, and DXT5 compression.  The compession this format uses is lossy (like
   JPEG) so expect a drop in quality with some images.  Also since the compession is
   lossy, you do not want to use compessed .dds with normal maps.  In order to compile
   this demo we will need to load the OpenGL extension ARB_texture_compression and
   get the function glCompressedTexImage2DARB().  This function is used in replacement
   to gluBuild2DMipmaps() and allows you to load a compressed image into OpenGL.
   These compressed images take up less memory and can run a bit faster.
*/


// Demo: .dds Loader.


#define WIN32_LEAN_AND_MEAN      // This trims down the libraries.
#define VC_LEANMEAN              // Trims even farther.


// All the necessary includes...
#include<windows.h>              // The windows header file.
#include<stdio.h>                // input/output.
#include<stdlib.h>               // Standard lib.
#include<gl/gl.h>                // Standard opengl include.
#include<gl/glu.h>               // Opengl utilities.
#include<gl/glext.h>             // OpenGL extensions.
#include"CDDSLoader.h"           // Has our dds class.


// Function Prototypes...
void RenderScene();
bool InitializeGL();
void Shutdown();


// And our global device context.  Used when swapping frames.
HDC g_HDC;

// This object holds all textures and texture related functions.
CDDSLoader Texture;

PFNGLCOMPRESSEDTEXIMAGE2DARBPROC glCompressedTexImage2DARB;


void SetupPixelFormat(HDC hDC)
{
   int nPixelFormat;

   static PIXELFORMATDESCRIPTOR pfd = {
         sizeof(PIXELFORMATDESCRIPTOR),   // size of structure.
         1,                               // always 1.
         PFD_DRAW_TO_WINDOW |             // support window
         PFD_SUPPORT_OPENGL |             // support OpenGl
         PFD_DOUBLEBUFFER,                // support double buffering
         PFD_TYPE_RGBA,                   // support RGBA
         16,                              // bit color mode
         0, 0, 0, 0, 0, 0,                // ignore color bits
         0,                               // no alpha buffer
         0,                               // ignore shift bit
         0,                               // no accumulation buffer
         0, 0, 0, 0,                      // ignore accumulation bits.
         16,                              // number of depth buffer bits.
         0,                               // number of stencil buffer bits.
         0,                               // 0 means no auxiliary buffer
         PFD_MAIN_PLANE,                  // The main drawing plane
         0,                               // this is reserved
         0, 0, 0 };                       // layer masks ignored.

   // this chooses the best pixel format and returns index.
   nPixelFormat = ChoosePixelFormat(hDC, &pfd);

   // This set pixel format to device context.
   SetPixelFormat(hDC, nPixelFormat, &pfd);
}


LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   static HGLRC hRC;                      // Rendering context.
   static HDC hDC;                        // Device context.
   int width, height;                     // The window width and height.

   switch(message)
      {
         case WM_CREATE:                  // Windows creation.
            hDC = GetDC(hwnd);            // This gets the device context for our window.
            g_HDC = hDC;                  // Assigns the global device context to this one.
            SetupPixelFormat(hDC);        // Call the pixel format function.

            hRC = wglCreateContext(hDC);  // Creates the rendering context.
            wglMakeCurrent(hDC, hRC);     // Makes the rendering context.
            return 0;
            break;

         case WM_CLOSE:                   // Close message.
         case WM_DESTROY:
            wglMakeCurrent(hDC, NULL);
            wglDeleteContext(hRC);        // Deletes the rendering context.

            PostQuitMessage(0);           // Says close the program.
            return 0;
            break;

         case WM_SIZE:                    // re-size message.
            height = HIWORD(lParam);      // This gets the height of the window.
            width = LOWORD(lParam);       // This gets the width of the window.

            if(height==0)                 // we don't want it to be possible for a
               {                          // height of 0.  If it is 0 me make it 1.
                  height = 1;
               }

            glViewport(0, 0, width, height);// resets the viewport to new dimensions.
            glMatrixMode(GL_PROJECTION);    // Sets the projection matrix.
            glLoadIdentity();               // Reset the modelview matrix.

            // calculate the aspect ratio of the window.
            gluPerspective(45.0f, (GLfloat)width/(GLfloat)height, 0.1f, 1000.0f);

            glMatrixMode(GL_MODELVIEW);     // Sets the projection matrix.
            glLoadIdentity();               // Reset the modelview matrix.

            return 0;
            break;


         case WM_KEYDOWN:
		      switch(wParam) 
		         {
                  case VK_ESCAPE:
                     PostQuitMessage(0);
                     break;
               }
            break;

         default:
            break;
      }
   
   // What this does is pass all of the unhandled messages to DefWindowProc
   return (DefWindowProc(hwnd, message, wParam, lParam));
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
   MSG msg;                                        // A message variable.
   WNDCLASSEX windowClass;                         // Your Window class.
   HWND hwnd;                                      // The Window handle.
   bool isFinished;                                // True then exit.

   // This is the Window class.
   windowClass.cbSize = sizeof(WNDCLASSEX);        // size of the WNDCLASSEX structure.
   windowClass.style = CS_HREDRAW | CS_VREDRAW;    // style of the window.
   windowClass.lpfnWndProc = WndProc;              // Address to the windows procedure.
   windowClass.cbClsExtra = 0;                     // Extra class information.
   windowClass.cbWndExtra = 0;                     // Extra window information.
   windowClass.hInstance = hInstance;              // Handle of application Instance.
   windowClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);// Handle of application Icon.
   windowClass.hCursor = LoadCursor(NULL, IDC_ARROW);// mouse cursor
   windowClass.hbrBackground = NULL;               // background color.
   windowClass.lpszMenuName = NULL;                // name of the main menu.
   windowClass.lpszClassName = "UltimateGameProgrammingClass";// window class name.
   windowClass.hIconSm = LoadIcon(NULL, IDI_APPLICATION);// icon when minimized.

   // You must register you class with Windows.
   if(!RegisterClassEx(&windowClass)) return 0;

   // Create the widnow.
   hwnd = CreateWindowEx(NULL,                                             // The extended style.
                        "UltimateGameProgrammingClass",                    // window class name.
                        "DDS Loading - Created by the Programming Ace.",   // window name.
                        WS_OVERLAPPEDWINDOW | WS_VISIBLE |                 // The window style.
                        WS_SYSMENU | WS_CLIPCHILDREN |                     // window style.
                        WS_CLIPSIBLINGS,                                   // window style.
                        100, 100,                                          // window x, y coord.
                        640,                                               // window width.
                        480,                                               // window height.
                        NULL,                                              // handle to parent.
                        NULL,                                              // menu.
                        hInstance,                                         // handle to app instance.
                        NULL);                                             // pointer to window data.
   
   // If there was an error with creating the window, then close the program.
   if(!hwnd) return 0;

   ShowWindow(hwnd, SW_SHOW); // This shows the window.
   UpdateWindow(hwnd);        // This forces a paint message.

   isFinished = false;        // False = running, True = not running.

   if(!InitializeGL()) isFinished = true;

   // This is the messsage loop.
   while(!isFinished)
      {
         if(PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
            {
               // If a quit message is received then stop rendering and quit the app.
               if(msg.message == WM_QUIT)
                  {
                     isFinished = true;
                  }

               TranslateMessage(&msg);  // Translate any messages.
               DispatchMessage(&msg);   // Dispatch any messages.
            }
         else
            {
               RenderScene();          // Render a frame.
            }
      }

   Shutdown();
   
   return (int)msg.wParam;
}


bool InitializeGL()
{
   glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

   // Set up OpenGL properties.
   glShadeModel(GL_SMOOTH);
   glEnable(GL_DEPTH_TEST);
   glEnable(GL_TEXTURE_2D);
   

   char *extensinos = (char*)glGetString(GL_EXTENSIONS);

   if(strstr(extensinos, "ARB_texture_compression") == NULL)
      return false;

   glCompressedTexImage2DARB = (PFNGLCOMPRESSEDTEXIMAGE2DARBPROC)wglGetProcAddress("glCompressedTexImage2DARB");
   if(!glCompressedTexImage2DARB) return false;


   // This will load all the textures into memory and make ready for use.
	if(!Texture.LoadDDSFile("image.dds")) return false;
	glGenTextures(1, &Texture.ID);
   glBindTexture(GL_TEXTURE_2D, Texture.ID);
	
	int w = Texture.imageWidth;
   int h = Texture.imageHeight;
   int mipSize;
   int mipOffset = 0;
	int mipFactor = 0;

   if(Texture.type == GL_COMPRESSED_RGBA_S3TC_DXT1_EXT) mipFactor = 8;
   else mipFactor = 16;

   // Here we manually load each mip map level of the image.
   for(int i = 0; i < Texture.totalMips; i++)
      {
         mipSize = ((w + 3) / 4) * ((h + 3) / 4) * mipFactor;

         glCompressedTexImage2DARB(GL_TEXTURE_2D, i, Texture.type,
                                 w, h, 0, mipSize, Texture.image + mipOffset);

         // Half the image size for the next mip-map level...
         w  = w  >> 1;
         h = h >> 1;

         // Move the offset to the next mip map.
         mipOffset += mipSize;
      }

   return true;
}


void RenderScene()
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);   // Clears the screen.
   glLoadIdentity();                   //Reset modelview matrix for new frame.

   // Move back 3 units.
   glTranslatef(0.0f, 0.0f, -3.0f);

   // Bind that texture to the texture variable.
   glBindTexture(GL_TEXTURE_2D, Texture.ID);

   // Draw the shape.
   glBegin(GL_QUADS);

      glTexCoord2f(0.0f, 0.0f); glVertex3f(-0.5f, -0.5f, 0.5f);
      glTexCoord2f(1.0f, 0.0f); glVertex3f(0.5f, -0.5f, 0.5f);
      glTexCoord2f(1.0f, 1.0f); glVertex3f(0.5f, 0.5f, 0.5f);
      glTexCoord2f(0.0f, 1.0f); glVertex3f(-0.5f, 0.5f, 0.5f);

   glEnd();

   SwapBuffers(g_HDC);
}


void Shutdown()
{
   glDeleteTextures(1, &Texture.ID);
   Texture.FreeImage();
}


// Recap:

// Loading .dds images can help a lot in controlling the
// memory requirements for the textures.  Using compressed images in OpenGL is actually
// faster but at a cost of image quality.  This is due to the lossy compession
// the dds format uses.  With that said it is not a good idea to use this with normal maps
// or the output will not be what you quite expect.


// Copyright October 2004
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com